using System;
using System.Web.UI;
using System.Web;
using System.Security.Permissions;
using System.ComponentModel;
using System.Text.RegularExpressions;

namespace Buayacorp.Web
{
	[
		ToolboxData("<{0}:CustomTextBox runat=\"server\" /></{0}:CustomTextBox>"),
		AspNetHostingPermission(SecurityAction.InheritanceDemand, Level = AspNetHostingPermissionLevel.Minimal),
		AspNetHostingPermission(SecurityAction.LinkDemand, Level = AspNetHostingPermissionLevel.Minimal)
	]
	public class CustomTextBox : System.Web.UI.WebControls.TextBox
	{
		#region Atributos
		private string characterExpression;
		private string validationExpression; 
		#endregion

		#region Propiedades
		[
			Bindable(true),
			Description("Expresin contra la cual se validar cada caracter que se ingrese en el TextBox. (Ej. [\\d])"),
			Category("Validacin")
		]
		public string CharacterExpression
		{
			get { return characterExpression; }
			set { characterExpression = value; }
		}

		[
			Bindable(true),
			Description("Expresin contra la cual se validar el valor que se ingrese en el TextBox"),
			Category("Validacin")
		]
		public string ValidationExpression
		{
			get { return validationExpression; }
			set { validationExpression = value; }
		} 
		
		public override string Text
		{
			set
			{
				if	(
					!string.IsNullOrEmpty(value) && 
				    !string.IsNullOrEmpty(validationExpression) && 
				    !Regex.IsMatch(value, validationExpression)
					)
					base.Text = string.Empty;
				else
					base.Text = value;
			}
		}
		#endregion

		#region Mtodos sobre-escritos
		protected override void AddAttributesToRender(HtmlTextWriter writer)
		{
			if (!string.IsNullOrEmpty(characterExpression))
				Utils.AddAttribute(this, "onkeypress",
								   string.Format("bc_ValidarCaracter(event, '{0}')",
												 characterExpression.Replace("\\", "\\\\").Replace("'", "\'")));

			if (!string.IsNullOrEmpty(validationExpression))
				Utils.AddAttribute(this, "onchange",
								   string.Format("bc_LimpiarValor(this, '{0}')",
												 validationExpression.Replace("\\", "\\\\").Replace("'", "\'")));

			base.AddAttributesToRender(writer);
		}

		protected override void OnPreRender(EventArgs e)
		{
			base.OnPreRender(e);
			if (!string.IsNullOrEmpty(characterExpression) || !string.IsNullOrEmpty(validationExpression))
				Page.ClientScript.RegisterClientScriptResource(GetType(), "Buayacorp.Web.Resources.script.js");
		} 
		#endregion

		#region Estado del control
		protected override void OnInit(EventArgs e)
		{
			Page.RegisterRequiresControlState(this);
			base.OnInit(e);
		}
		protected override object SaveControlState()
		{
			Pair state = new Pair();
			state.First = base.SaveControlState();

			state.Second = new string[] { characterExpression, validationExpression };

			return state;
		}

		protected override void LoadControlState(object state)
		{
			Pair p = state as Pair;
			if (p != null)
			{
				base.LoadControlState(p.First);

				string[] v = p.Second as string[];

				if (v != null)
				{
					characterExpression = v[0];
					validationExpression = v[1];
				}
			}
		} 
		#endregion
	}
}
